// vMac 0.1 Source
// By Philip Cummins (PC)
// Copyright PC 1997
   
// PC - IWM Function Files

#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "IWM.h"
#	ifdef _IWM_Debug
#include <stdio.h>
#	endif

UBYTE vSel; // Selection Line from VIA

IWM_Ty IWM;

int  IWM_Init (void)
{
  IWM.DataIn = IWM.Handshake = IWM.Status = IWM.Mode = IWM.DataOut = IWM.Lines = 0;

  return 0;
}

void  IWM_Reset (void)
{
  (void) IWM_Init();
}

void IWM_Set_Lines (UBYTE line, Mode_Ty mode)
{
  if (mode == Off)
    { IWM.Lines &= (0xFF - line); }
  else
    { IWM.Lines |= line; }
}

UBYTE IWM_GetB (CPTR addr)
{
  UBYTE value = 0;
  
  if ((addr & 1) == 0)
  {
#	ifdef _IWM_Debug
    printf("IWM GB %4lx\n", addr);
#	endif
    switch (addr >> 9)
	{
      case kph0L :
        IWM_Set_Lines(kph0, Off); return 0; break;
      case kph0H :
        IWM_Set_Lines(kph0, On);  return 0; break;
      case kph1L :
        IWM_Set_Lines(kph1, Off); return 0; break;
      case kph1H :
        IWM_Set_Lines(kph1, On);  return 0; break;
      case kph2L :
        IWM_Set_Lines(kph2, Off); return 0; break;
      case kph2H :
        IWM_Set_Lines(kph2, On);  return 0; break;
      case kph3L :
        IWM_Set_Lines(kph3, Off); return 0; break;
      case kph3H :
        IWM_Set_Lines(kph3, On);  return 0; break;
      case kmtrOff :
        IWM_Set_Lines(kmtr, Off); 
        IWM.Status &= 0xDF;       return 0; break;
      case kmtrOn :
        IWM_Set_Lines(kmtr, On);  
        IWM.Status |= 0x20;       return 0; break;
      case kintDrive :
        IWM_Set_Lines(kdrv, Off); return 0; break;
      case kextDrive :
        IWM_Set_Lines(kdrv, On);  return 0; break;
      case kq6L :
        IWM_Set_Lines(kq6, Off);  return 0; break;
      case kq6H :
        IWM_Set_Lines(kq6, On);   return 0; break;
      case kq7L :
        value = IWM_Read_Reg();
        IWM_Set_Lines(kq7, Off);  
        return value; break;
      case kq7H :
        IWM_Set_Lines(kq7, On);   return 0; break;
	}
  }
  else
  {
    return 0;
  }

  return 0;

}

UWORD IWM_GetW (CPTR addr)
{
  return 0;
}

ULONG IWM_GetL (CPTR addr)
{
  return 0;
}

void  IWM_PutB (CPTR addr, UBYTE in)
{
  if ((addr & 1) == 0)
  {
#	ifdef _IWM_Debug
    printf("IWM PB %4lx %2x\n", addr, in);
#	endif
    switch (addr >> 9)
	{
      case kph0L :
        IWM_Set_Lines(kph0, Off); break;
      case kph0H :
        IWM_Set_Lines(kph0, On);  break;
      case kph1L :
        IWM_Set_Lines(kph1, Off); break;
      case kph1H :
        IWM_Set_Lines(kph1, On);  break;
      case kph2L :
        IWM_Set_Lines(kph2, Off); break;
      case kph2H :
        IWM_Set_Lines(kph2, On);  break;
      case kph3L :
        IWM_Set_Lines(kph3, Off); break;
      case kph3H :
        IWM_Set_Lines(kph3, On);  break;
      case kmtrOff :
        IWM_Set_Lines(kmtr, Off); 
        IWM.Status &= 0xDF;       break;
      case kmtrOn :
        IWM_Set_Lines(kmtr, On);  
        IWM.Status |= 0x20;       break;
      case kintDrive :
        IWM_Set_Lines(kdrv, Off); break;
      case kextDrive :
        IWM_Set_Lines(kdrv, On);  break;
      case kq6L :
        IWM_Set_Lines(kq6, Off);  break;
      case kq6H :
        IWM_Set_Lines(kq6, On);   break;
      case kq7L :
        IWM_Set_Lines(kq7, Off);  break;
      case kq7H :
        IWM_Write_Reg(in);
        IWM_Set_Lines(kq7, On);   break;
	}
  }
}

void  IWM_PutW (CPTR addr, UWORD in)
{
	;
}

void  IWM_PutL (CPTR addr, ULONG in)
{
	;
}

UBYTE IWM_Read_Reg (void)
{
  switch ((IWM.Lines & (kq6 + kq7)) >> 6)
  {
    case 0 :
#	ifdef _IWM_Debug
      printf("IWM Data Read\n");
#	endif
      return IWM.DataIn; break;
    case 1 :
#	ifdef _IWM_Debug
      printf("IWM Status Read\n");
#	endif
      return IWM.Status; break;
    case 2 : 
#	ifdef _IWM_Debug
      printf("IWM Handshake Read\n");
#	endif
      return IWM.Handshake; break;
    case 3 :
      return 0; break;
  }

  return 0;

}

void IWM_Write_Reg (UBYTE in)
{
  if (((IWM.Lines & kmtr) >> 4) == 0)
  {
#	ifdef _IWM_Debug
    printf("IWM Mode Register Write\n");
#	endif
    IWM.Mode = in;
    IWM.Status = ((IWM.Status & 0xE0) + (IWM.Mode & 0x1F));
  }
}

void IWM_Update (void)
{
	;
}

// VIA Interface Headers

UBYTE VIA_GORA5 (void) // Floppy Disk Line SEL
{
#	ifdef _VIA_Interface_Debug
  printf("VIA ORA5 attempts to be an input\n");
#	endif
  return 0;
}

void  VIA_PORA5 (UBYTE Data)
{
  vSel = Data;
}
