// vMac 0.1 Source
// By Philip Cummins (PC)
// Copyright PC 1997

// PC - Memory Mapping Code

#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "options.h"
#include "Memory.h"
#include "custom.h"
#include "newcpu.h"
#include "readcpu.h"

#	ifdef _Memory_Debug
#include <stdio.h>
#	endif

#include "IWM.h"
#include "RAM.h"
#include "ROM.h"
#include "SCC.h"
#include "SCSI.h"
#include "VIA.h"

UBYTE vOverlay = 1;

UBYTE buserr = 0;

int Memory_Init (void)
{
  vOverlay = 1;
  return 0;
}

void  Memory_Reset (void)
{
  (void) Memory_Init();
}

ULONG Get_MM (CPTR addr, Size_Ty Size)
{
  CPTR taddr = addr & kMemory_Extent;

  if (vOverlay)
  {
    switch (taddr >> 22)
    {
      case 0 : // ROM
        if (taddr < kROM_Overlay_Top)
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) ROM_GetB (taddr % kROM_Size); break;
            case Size_Word : return (UWORD) ROM_GetW (taddr % kROM_Size); break;
            case Size_Long : return (ULONG) ROM_GetL (taddr % kROM_Size); break;
          }
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
        return 0;
        break;
      case 1 : // ROM, SCSI and RAM
        if ((taddr >= kROM_Base) && (taddr < kROM_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) ROM_GetB ((taddr - kROM_Base) % kROM_Size); break;
            case Size_Word : return (UWORD) ROM_GetW ((taddr - kROM_Base) % kROM_Size); break;
            case Size_Long : return (ULONG) ROM_GetL ((taddr - kROM_Base) % kROM_Size); break;
          }
        }
        if ((taddr >= kRAM_Overlay_Base) && (taddr < kRAM_Overlay_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) RAM_GetB ((taddr - kRAM_Overlay_Base) % kRAM_Size); break;
            case Size_Word : return (UWORD) RAM_GetW ((taddr - kRAM_Overlay_Base) % kRAM_Size); break;
            case Size_Long : return (ULONG) RAM_GetL ((taddr - kRAM_Overlay_Base) % kRAM_Size); break;
          }
        }
        if ((taddr >= kSCSI_Block_Base) && (taddr < kSCSI_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) SCSI_GetB (taddr - kSCSI_Block_Base); break;
            case Size_Word : return (UWORD) SCSI_GetW (taddr - kSCSI_Block_Base); break;
            case Size_Long : return (ULONG) SCSI_GetL (taddr - kSCSI_Block_Base); break;
          }
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
        return 0;
        break;
      case 2 : // SCC
        if ((taddr >= kSCCRd_Block_Base) && (taddr < kSCCRd_Block_Top)) // Read Only taddress
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) SCC_GetB ((taddr - kSCCRdBase) % kSCC_Size); break;
            case Size_Word : return (UWORD) SCC_GetW ((taddr - kSCCRdBase) % kSCC_Size); break;
            case Size_Long : return (ULONG) SCC_GetL ((taddr - kSCCRdBase) % kSCC_Size); break;
          }
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
		return 0;
        break;
      case 3 : // IWM and VIA
        if ((taddr >= kVIA_Block_Base) && (taddr < kVIA_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) VIA_GetB ((taddr - kVIA_Base) % kVIA_Size); break;
            case Size_Word : return (UWORD) VIA_GetW ((taddr - kVIA_Base) % kVIA_Size); break;
            case Size_Long : return (ULONG) VIA_GetL ((taddr - kVIA_Base) % kVIA_Size); break;
          }
        }
        if ((taddr >= kIWM_Block_Base) && (taddr < kIWM_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) IWM_GetB ((taddr - kIWM_Base) % kIWM_Size); break;
            case Size_Word : return (UWORD) IWM_GetW ((taddr - kIWM_Base) % kIWM_Size); break;
            case Size_Long : return (ULONG) IWM_GetL ((taddr - kIWM_Base) % kIWM_Size); break;
          }
        }
        if ((taddr >= kAutoVector_Base) && (taddr < kAutoVector_Top))
        { 
          Exception(regs.intmask+24);
          return 0;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
        return 0;
        break;
    }
  }
  else
  {
    switch (taddr >> 22)
    {
      case 0 : // RAM
        if (taddr < kRAM_Top)
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) RAM_GetB (taddr % kRAM_Size); break;
            case Size_Word : return (UWORD) RAM_GetW (taddr % kRAM_Size); break;
            case Size_Long : return (ULONG) RAM_GetL (taddr % kRAM_Size); break;
          }
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
        return 0;
        break;
      case 1 : // ROM and SCSI
        if ((taddr >= kROM_Base) && (taddr < kROM_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) ROM_GetB ((taddr - kROM_Base) % kROM_Size); break;
            case Size_Word : return (UWORD) ROM_GetW ((taddr - kROM_Base) % kROM_Size); break;
            case Size_Long : return (ULONG) ROM_GetL ((taddr - kROM_Base) % kROM_Size); break;
          }
        }
        if ((taddr >= kSCSI_Block_Base) && (taddr < kSCSI_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) SCSI_GetB (taddr - kSCSI_Block_Base); break;
            case Size_Word : return (UWORD) SCSI_GetW (taddr - kSCSI_Block_Base); break;
            case Size_Long : return (ULONG) SCSI_GetL (taddr - kSCSI_Block_Base); break;
          }
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
        return 0;
        break;
      case 2 : // SCC
        if ((taddr >= kSCCRd_Block_Base) && (taddr < kSCCRd_Block_Top)) // Read Only taddress
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) SCC_GetB ((taddr - kSCCRdBase) % kSCC_Size); break;
            case Size_Word : return (UWORD) SCC_GetW ((taddr - kSCCRdBase) % kSCC_Size); break;
            case Size_Long : return (ULONG) SCC_GetL ((taddr - kSCCRdBase) % kSCC_Size); break;
          }
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
	return 0;
        break;
      case 3 : // IWM and VIA
        if ((taddr >= kVIA_Block_Base) && (taddr < kVIA_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) VIA_GetB ((taddr - kVIA_Base) % kVIA_Size); break;
            case Size_Word : return (UWORD) VIA_GetW ((taddr - kVIA_Base) % kVIA_Size); break;
            case Size_Long : return (ULONG) VIA_GetL ((taddr - kVIA_Base) % kVIA_Size); break;
          }
        }
        if ((taddr >= kIWM_Block_Base) && (taddr < kIWM_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : return (UBYTE) IWM_GetB ((taddr - kIWM_Base) % kIWM_Size); break;
            case Size_Word : return (UWORD) IWM_GetW ((taddr - kIWM_Base) % kIWM_Size); break;
            case Size_Long : return (ULONG) IWM_GetL ((taddr - kIWM_Base) % kIWM_Size); break;
          }
        }
        if ((taddr >= kAutoVector_Base) && (taddr < kAutoVector_Top))
        {
          Exception(regs.intmask+24);
/* Added 0 -Michael */
          return 0;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx\n", taddr); break;
          case Size_Word : printf("Word Access at %8lx\n", taddr); break;
          case Size_Long : printf("Long Access at %8lx\n", taddr); break;
        }
#	endif
        return 0;
        break;
    }
  }

  return 0;

}

void  Put_MM (CPTR addr, Size_Ty Size, ULONG in)
{
  CPTR taddr = addr & kMemory_Extent;

  if (vOverlay)
  {
    switch (taddr >> 22)
    {
      case 0 : // ROM
        if (taddr < kROM_Overlay_Top)
        {
          switch (Size)
          {
            case Size_Byte : ROM_PutB (taddr % kROM_Size, (UBYTE) in); break;
            case Size_Word : ROM_PutW (taddr % kROM_Size, (UWORD) in); break;
            case Size_Long : ROM_PutL (taddr % kROM_Size, (ULONG) in); break;
          }
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
      case 1 : // ROM, SCSI and RAM
        if ((taddr >= kROM_Base) && (taddr < kROM_Top))
        {
          switch (Size)
          {
            case Size_Byte : ROM_PutB ((taddr - kROM_Base) % kROM_Size, (UBYTE) in); break;
            case Size_Word : ROM_PutW ((taddr - kROM_Base) % kROM_Size, (UWORD) in); break;
            case Size_Long : ROM_PutL ((taddr - kROM_Base) % kROM_Size, (ULONG) in); break;
          }
          return;
        }
        if ((taddr >= kRAM_Overlay_Base) && (taddr < kRAM_Overlay_Top))
        {
          switch (Size)
          {
            case Size_Byte : RAM_PutB ((taddr - kRAM_Overlay_Base) % kRAM_Size, (UBYTE) in); break;
            case Size_Word : RAM_PutW ((taddr - kRAM_Overlay_Base) % kRAM_Size, (UWORD) in); break;
            case Size_Long : RAM_PutL ((taddr - kRAM_Overlay_Base) % kRAM_Size, (ULONG) in); break;
          }
          return;
        }
        if ((taddr >= kSCSI_Block_Base) && (taddr < kSCSI_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : SCSI_PutB (taddr - kSCSI_Block_Base, (UBYTE) in); break;
            case Size_Word : SCSI_PutW (taddr - kSCSI_Block_Base, (UWORD) in); break;
            case Size_Long : SCSI_PutL (taddr - kSCSI_Block_Base, (ULONG) in); break;
          }
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
      case 2 : // SCC
        if ((taddr >= kSCCWr_Block_Base) && (taddr < kSCCWr_Block_Top)) // Write Only taddress
        {
          switch (Size)
          {
            case Size_Byte : SCC_PutB ((taddr - kSCCWrBase) % kSCC_Size, (UBYTE) in); break;
            case Size_Word : SCC_PutW ((taddr - kSCCWrBase) % kSCC_Size, (UWORD) in); break;
            case Size_Long : SCC_PutL ((taddr - kSCCWrBase) % kSCC_Size, (ULONG) in); break;
          }
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
      case 3 : // IWM and VIA
        if ((taddr >= kVIA_Block_Base) && (taddr < kVIA_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : VIA_PutB ((taddr - kVIA_Base) % kVIA_Size, (UBYTE) in); break;
            case Size_Word : VIA_PutW ((taddr - kVIA_Base) % kVIA_Size, (UWORD) in); break;
            case Size_Long : VIA_PutL ((taddr - kVIA_Base) % kVIA_Size, (ULONG) in); break;
          }
          return;
        }
        if ((taddr >= kIWM_Block_Base) && (taddr < kIWM_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : IWM_PutB ((taddr - kIWM_Base) % kIWM_Size, (UBYTE) in); break;
            case Size_Word : IWM_PutW ((taddr - kIWM_Base) % kIWM_Size, (UWORD) in); break;
            case Size_Long : IWM_PutL ((taddr - kIWM_Base) % kIWM_Size, (ULONG) in); break;
          }
          return;
        }
        if ((taddr >= kAutoVector_Base) && (taddr < kAutoVector_Top))
        {
          Exception(regs.intmask+24);
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
    }
  }
  else
  {
    switch (taddr >> 22)
    {
      case 0 : // RAM
        if (taddr < kRAM_Top)
        {
          switch (Size)
          {
            case Size_Byte : RAM_PutB (taddr % kRAM_Size, (UBYTE) in); break;
            case Size_Word : RAM_PutW (taddr % kRAM_Size, (UWORD) in); break;
            case Size_Long : RAM_PutL (taddr % kRAM_Size, (ULONG) in); break;
          }
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
      case 1 : // ROM and SCSI
        if ((taddr >= kROM_Base) && (taddr < kROM_Top))
        {
          switch (Size)
          {
            case Size_Byte : ROM_PutB ((taddr - kROM_Base) % kROM_Size, (UBYTE) in); break;
            case Size_Word : ROM_PutW ((taddr - kROM_Base) % kROM_Size, (UWORD) in); break;
            case Size_Long : ROM_PutL ((taddr - kROM_Base) % kROM_Size, (ULONG) in); break;
          }
          return;
        }
        if ((taddr >= kSCSI_Block_Base) && (taddr < kSCSI_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : SCSI_PutB (taddr - kSCSI_Block_Base, (UBYTE) in); break;
            case Size_Word : SCSI_PutW (taddr - kSCSI_Block_Base, (UWORD) in); break;
            case Size_Long : SCSI_PutL (taddr - kSCSI_Block_Base, (ULONG) in); break;
          }
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
      case 2 : // SCC
        if ((taddr >= kSCCWr_Block_Base) && (taddr < kSCCWr_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : SCC_PutB ((taddr - kSCCWrBase) % kSCC_Size, (UBYTE) in); break;
            case Size_Word : SCC_PutW ((taddr - kSCCWrBase) % kSCC_Size, (UWORD) in); break;
            case Size_Long : SCC_PutL ((taddr - kSCCWrBase) % kSCC_Size, (ULONG) in); break;
          }
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
      case 3 : // IWM and VIA
	if ((taddr >= kVIA_Block_Base) && (taddr < kVIA_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : VIA_PutB ((taddr - kVIA_Base) % kVIA_Size, (UBYTE) in); break;
            case Size_Word : VIA_PutW ((taddr - kVIA_Base) % kVIA_Size, (UWORD) in); break;
            case Size_Long : VIA_PutL ((taddr - kVIA_Base) % kVIA_Size, (ULONG) in); break;
          }
          return;
        }
        if ((taddr >= kIWM_Block_Base) && (taddr < kIWM_Block_Top))
        {
          switch (Size)
          {
            case Size_Byte : IWM_PutB ((taddr - kIWM_Base) % kIWM_Size, (UBYTE) in); break;
            case Size_Word : IWM_PutW ((taddr - kIWM_Base) % kIWM_Size, (UWORD) in); break;
            case Size_Long : IWM_PutL ((taddr - kIWM_Base) % kIWM_Size, (ULONG) in); break;
          }
          return;
        }
        if ((taddr >= kAutoVector_Base) && (taddr < kAutoVector_Top))
        {
          Exception(regs.intmask+24);
          return;
        }
#	ifdef _Memory_Debug
        switch (Size)
        {
          case Size_Byte : printf("Byte Access at %8lx, %2lx\n", taddr, (UBYTE) in); break;
          case Size_Word : printf("Word Access at %8lx, %4lx\n", taddr, (UWORD) in); break;
          case Size_Long : printf("Long Access at %8lx, %8lx\n", taddr, (ULONG) in); break;
        }
#	endif
        break;
    }
  }
}

// UAE 0.6 Emulation Direct Memory Access Function

UWORD *get_real_address (CPTR addr)
{
  CPTR taddr = addr & kMemory_Extent;
  
  if (vOverlay)
  {
    switch (taddr >> 22)
    {
      case 0 : // ROM
        if (taddr < kROM_Overlay_Top)
        {
          return ROM_Xlate (taddr % kROM_Size);
        }
		return NULL;
        break;
      case 1 : // ROM, SCSI and RAM
        if ((taddr >= kROM_Base) && (taddr < kROM_Top))
        {
          return ROM_Xlate ((taddr - kROM_Base) % kROM_Size);
        }
        if ((taddr >= kRAM_Overlay_Base) && (taddr < kRAM_Overlay_Top))
        {
          return RAM_Xlate ((taddr - kRAM_Overlay_Base) % kRAM_Size);
        }
        return NULL;
        break;
      case 2 : // SCC
        return NULL;
        break;
      case 3 : // IWM and VIA
        return NULL;
        break;
    }
  }
  else
  {
    switch (taddr >> 22)
    {
      case 0 : // RAM
        if (taddr < kRAM_Top)
        {
          return RAM_Xlate (taddr % kRAM_Size);
        }
		return NULL;
        break;
      case 1 : // ROM and SCSI
        if ((taddr >= kROM_Base) && (taddr < kROM_Top))
        {
          return ROM_Xlate ((taddr - kROM_Base) % kROM_Size);
        }
        return NULL;
        break;
      case 2 : // SCC
        return NULL;
        break;
      case 3 : // IWM and VIA
        return NULL;
        break;
    }
    return 0;
  }
  return 0;
}

// UAE 0.6 68K Emulator Memory Access Functions

ULONG get_long(CPTR addr)
{
  return (ULONG) Get_MM(addr & kMemory_Extent,Size_Long);
}

UWORD get_word(CPTR addr)
{
  return (UWORD) Get_MM(addr & kMemory_Extent,Size_Word);
}

UBYTE get_byte(CPTR addr)
{
  return (UBYTE) Get_MM(addr & kMemory_Extent,Size_Byte);
}

void put_long(CPTR addr, ULONG in)
{
  Put_MM(addr & kMemory_Extent, Size_Long, (ULONG) in);
}

void put_word(CPTR addr, UWORD in)
{
  Put_MM(addr & kMemory_Extent, Size_Word, (ULONG) in);
}

void put_byte(CPTR addr, UBYTE in)
{
  Put_MM(addr & kMemory_Extent, Size_Byte, (ULONG) in);
}

// VIA Interface Functions

UBYTE VIA_GORA4 (void) // Overlay/Normal Memory Mapping
{
#	ifdef _VIA_Interface_Debug
  	printf("VIA ORA4 attempts to be an input\n");
#	endif
  	return 0;
}

void  VIA_PORA4 (UBYTE Data)
{
  vOverlay = Data;
}
