// vMac 0.x Source
// By Philip Cummins (PC)
// Copyright PC 1997
   
// PC - RTC (Real Time Clock) Functions

#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "RTC.h"
#include <time.h>
#	ifdef _RTC_Debug
#include <stdio.h>
#	endif

RTC_Ty RTC;

// RTC Functions

void Set_RTC_Time (void) {

	struct tm Time;
	time_t Current_Time = 0;
	time_t Start_Time = 0;

	Time.tm_sec = 0;
	Time.tm_min = 0;
	Time.tm_hour = 1;  // 1 AM. Not 12:00 AM
	Time.tm_mday = 1;
	Time.tm_mon = 0;
	Time.tm_year = 4;

	Start_Time = mktime(&Time);

	Current_Time = time(NULL);
	Current_Time -= Start_Time;

#	ifndef _MacOS
//	Current_Time+=2082823200;
#	endif

	RTC.Seconds_1[0] = Current_Time & 0xFF;
	RTC.Seconds_1[1] = (Current_Time & 0xFF00) >> 8;
	RTC.Seconds_1[2] = (Current_Time & 0xFF0000) >> 16;
	RTC.Seconds_1[3] = (Current_Time & 0xFF000000) >> 24;

	RTC.Seconds_2[0] = RTC.Seconds_1[0];
	RTC.Seconds_2[1] = RTC.Seconds_1[1];
	RTC.Seconds_2[2] = RTC.Seconds_1[2];
	RTC.Seconds_2[3] = RTC.Seconds_1[3];

}
/*
void Set_RTC_Time (void) {

	struct tm Time;

	time_t Current_Time = 0;
	time_t RealTime = time(NULL);

        Time = *localtime(&RealTime);

	printf("timezone secs: %d\n", timezone);
	printf("timezone hours: %d\n", timezone / 60 / 60);
	printf("daylight: %d\n", daylight);

	Current_Time = mktime(&Time);

#	ifndef _MacOS
	Current_Time+=2082823200;
#	endif

	RTC.Seconds_1[0] = Current_Time & 0xFF;
	RTC.Seconds_1[1] = (Current_Time & 0xFF00) >> 8;
	RTC.Seconds_1[2] = (Current_Time & 0xFF0000) >> 16;
	RTC.Seconds_1[3] = (Current_Time & 0xFF000000) >> 24;

	RTC.Seconds_2[0] = RTC.Seconds_1[0];
	RTC.Seconds_2[1] = RTC.Seconds_1[1];
	RTC.Seconds_2[2] = RTC.Seconds_1[2];
	RTC.Seconds_2[3] = RTC.Seconds_1[3];

}
*/

int   RTC_Init (void)
{
  int Counter = 0;

  RTC.Enabled = RTC.Clock = RTC.Mode = RTC.Command = RTC.Data = RTC.Counter = 0;  
  for (Counter = 0; Counter <= 3; Counter++)
    { RTC.Seconds_1[Counter] = RTC.Seconds_2[Counter] = 0; }
  for (Counter = 0; Counter <= 19; Counter++)
    { RTC.RAM[Counter] = 0; }

  Set_RTC_Time();
  
  return RTC_Load();
}

void  RTC_Reset (void)
{
	;
}

void RTC_Interrupt (void)
{
  long Seconds = 0;
  
  Seconds = (RTC.Seconds_1[3] << 24) + (RTC.Seconds_1[2] << 16) + (RTC.Seconds_1[1] << 8) + RTC.Seconds_1[0];
  Seconds++;
  RTC.Seconds_1[0] = Seconds & 0xFF;
  RTC.Seconds_1[1] = (Seconds & 0xFF00) >> 8;
  RTC.Seconds_1[2] = (Seconds & 0xFF0000) >> 16;
  RTC.Seconds_1[3] = (Seconds & 0xFF000000) >> 24;
  
  Seconds = (RTC.Seconds_2[3] << 24) + (RTC.Seconds_2[2] << 16) + (RTC.Seconds_2[1] << 8) + RTC.Seconds_2[0];
  Seconds++;
  RTC.Seconds_2[0] = Seconds & 0xFF;
  RTC.Seconds_2[1] = (Seconds & 0xFF00) >> 8;
  RTC.Seconds_2[2] = (Seconds & 0xFF0000) >> 16;
  RTC.Seconds_2[3] = (Seconds & 0xFF000000) >> 24;
}

void  RTC_Put (UBYTE in)
{
  RTC.Data = (RTC.Data << 1) + in;
  RTC.Counter++;
  if (RTC.Counter == 8)
  {
    RTC_DoCmd();
  }
}

UBYTE RTC_Get  (void)
{
  return ((RTC.Data >> --RTC.Counter) & 0x01);
}

void RTC_DoCmd (void)
{
  if (RTC.Mode == 0) // This Byte is a RTC Command
  {
    RTC.Command = RTC.Data;
    RTC.Data = 0;
    if ((RTC.Command & 0x80) == 0x00) // Write Command
    {
      RTC.Mode = 1; // Set to switch to Write Mode
      RTC.Counter = 0;  // Reset the Counter
    }
    else // Read Command
    {
      RTC_Read_Reg(); // Read RTC Register
    }
  }
  else // Data has come in that needs to be written
  {
    RTC_Write_Reg();
  }
}

void RTC_Read_Reg (void)
{
# ifdef _RTC_Debug
  printf("RTC Read Reg %2x\n", RTC.Command);
# endif
  switch ((RTC.Command & 0x7C) >> 2)
  {
    case 0 :
      RTC.Data = RTC.Seconds_1[0]; break;
    case 1 :
      RTC.Data = RTC.Seconds_1[1]; break;
    case 2 :
      RTC.Data = RTC.Seconds_1[2]; break;
    case 3 :
      RTC.Data = RTC.Seconds_1[3]; break;
    case 4 :
	  RTC.Data = RTC.Seconds_2[0]; break; 
	case 5 :
	  RTC.Data = RTC.Seconds_2[1]; break; 
	case 6 :
	  RTC.Data = RTC.Seconds_2[2]; break; 
	case 7 :
	  RTC.Data = RTC.Seconds_2[3]; break; 
    case 8 : case 9 : case 10 : case 11 :
      RTC.Data = RTC.RAM[((RTC.Command & 0x0C) >> 2) + 0x10]; break;
    case 16 : case 17 : case 18 : case 19 :
    case 20 : case 21 : case 22 : case 23 :
    case 24 : case 25 : case 26 : case 27 :
    case 28 : case 29 : case 30 : case 31 :
      RTC.Data = RTC.RAM[((RTC.Command & 0x3C) >> 2)]; break;
    default :
      RTC.Data = 0;
#     ifdef _RTC_Debug
      printf("RTC Read Reg Unknown, %2x\n", RTC.Command);
#     endif
      break;
  }
  RTC.Command = RTC.Mode = 0;
}

void RTC_Write_Reg (void)
{
# ifdef _RTC_Debug
  printf("RTC Write Reg %2x %2x\n", RTC.Command, RTC.Data);
# endif
  switch ((RTC.Command & 0x7C) >> 2)
  {
    case 0 :
      RTC.Seconds_1[0] = RTC.Data; break;
    case 1 :
      RTC.Seconds_1[1] = RTC.Data; break;
    case 2 :
      RTC.Seconds_1[2] = RTC.Data; break;
    case 3 :
      RTC.Seconds_1[3] = RTC.Data; break;
    case 4 :
      RTC.Seconds_2[0] = RTC.Data; break;
    case 5 :
      RTC.Seconds_2[1] = RTC.Data; break;
    case 6 :
      RTC.Seconds_2[2] = RTC.Data; break;
    case 7 :
      RTC.Seconds_2[3] = RTC.Data; break;
    case 8 : case 9 : case 10 : case 11 :
      RTC.RAM[((RTC.Command & 0x0C) >> 2) + 0x10] = RTC.Data; break;
    case 12 :
      break; // Test Write, do nothing
    case 13 : 
      RTC_Save(); break; // Write_Protect Register
    case 16 : case 17 : case 18 : case 19 :
    case 20 : case 21 : case 22 : case 23 :
    case 24 : case 25 : case 26 : case 27 :
    case 28 : case 29 : case 30 : case 31 :
      RTC.RAM[((RTC.Command & 0x3C) >> 2)] = RTC.Data; break;
    default :
      RTC.Data = 0;
#	ifdef _RTC_Debug
      printf("RTC Write Reg Unknown, %2x\n", RTC.Command);
#	endif
      break;
  }
  RTC.Command = RTC.Data = RTC.Counter = RTC.Mode = 0;
}

UBYTE RTC_Load (void)
{
  FILE *PRAM_File;
  int  File_Size = 0;
  
  PRAM_File = fopen("vMac.PRAM", "rb");
  if (PRAM_File == NULL)
  {
#	ifdef _RTC_Debug
    	printf("The vMac.PRAM could not be found, will create one.\n");
#	endif
  }
  else
  {
    File_Size = fread(RTC.RAM, 20, 1, PRAM_File);
    if (File_Size != 1)
    {
#	ifdef _RTC_Debug
      	printf("The vMac.PRAM is not 20 Bytes large.\n");
#	endif
      fclose(PRAM_File);
      return 1;
    }
    fclose(PRAM_File);
#	ifdef _RTC_Debug
	printf("vMac.PRAM loaded successfully.\n");
#	endif
  }
  return 0;
}

void RTC_Save (void)
{
  FILE *PRAM_File;
  int File_Size = 0;
  
  PRAM_File = fopen("vMac.PRAM", "wb");
  File_Size = fwrite(RTC.RAM, 20, 1, PRAM_File);
  if (File_Size != 1)
  {
#	ifdef _RTC_Debug
    	printf("vMac had problems writing the PRAM file\n");
#	endif
  }
  fclose(PRAM_File);
}

// VIA Interface Functions

UBYTE VIA_GORB2 (void) // RTC Enable
{
#	ifdef _VIA_Interface_Debug
  	printf("VIA ORB2 attempts to be an input\n");
#	endif
  	return 0;
}

void  VIA_PORB2 (UBYTE Data)
{
  RTC.Enabled = Data;
}

UBYTE VIA_GORB1 (void) // RTC Data Clock
{
#	ifdef _VIA_Interface_Debug
  	printf("VIA ORB1 attempts to be an input\n");
#	endif
	return 0;
}

void  VIA_PORB1 (UBYTE Data)
{
   RTC.Clock = Data;
}

UBYTE VIA_GORB0 (void) // RTC Data
{
  if ((RTC.Enabled == 0) && (RTC.Clock == 1))
    { return RTC_Get(); }
  else
    { return 0; }
}

void  VIA_PORB0 (UBYTE Data)
{
  if ((RTC.Enabled == 0) && (RTC.Clock == 1))
    { RTC_Put(Data); }
}
