// PC - Sony Driver Headers

// Constants

// Sony_Table Indexes

#define kOpen    0
#define kPrime   1
#define kControl 2
#define kStatus  3
#define kClose   4

// Sony Variable Drive Offsets

#define kDrive1 74	   // Location of Drive 1 Variables
#define kDrive2 140	   // Location of Drive 2 Variables

// Sony Variable Drive Setting Offsets

#define kTrack       0 // Current Track
#define kWriteProt   2 // FF if Write Protected, 00 if readable
#define kDiskInPlace 3 // 00 = No Disk, 01 = Disk In, 2 = MacOS Read, FC-FF = Just Ejected
#define kInstalled   4 // 00 = Unknown, 01 = Installed, FF = Not Installed
#define kSides       5 // 00 if Single Sided Drive, FF if Doubled Sided Drive
#define kQLink       6 // Link to Next Drive
#define kQType      10 // Drive Type (0 = Size Saved, 1 = Very Large)
#define kQDriveNo   12 // Drive Number (1 = Internal, 2 = External)
#define kQRefNum    14 // Driver Reference Number (-5 for .Sony, FFFB)
#define kQFSID      16 // File System ID (0 = MacOS)
#define kTwoSideFmt 18 // FF if double-sided format, 00 if single-sided format
#define kNewIntf    19 // FF if new 800K interface or 00 if old 400K interface
#define kDriveErrs  20 // Drive Soft Errors

// Sony Driver Control Call csCodes

#define kKillIO             1
#define kVerifyDisk         5
#define kFormatDisk         6
#define kEjectDisk          7
#define kSetTagBuffer       8
#define kTrackCacheControl  9
#define kReturnDriveInfo   23

// Sony Driver Status Call csCodes

#define kDriveStatus  8

// Parameter Block Offsets

#define kqLink         0
#define kqType         4
#define kioTrap        6
#define kioCmdAddr     8
#define kioCompletion 12
#define kioResult     16
#define kioNamePtr    18
#define kioVRefNum    22
#define kioRefNum     24
#define kcsCode       26
#define kcsParam      28
#define kioBuffer     32 // Buffer to store data into
#define kioReqCount   36 // Requested Number of Bytes
#define kioActCount   40 // Actual Number of Bytes obtained
#define kioPosMode    44 // Positioning Mode
#define kioPosOffset  46 // Position Offset

// Positioning Modes

#define kfsAtMark    0 // At Mark (Ignore PosOffset)
#define kfsFromStart 1 // At Start (PosOffset is absolute)
#define kfsFromLEOF  2 // From Logical End of File - PosOffset
#define kfsFromMark  3 // At Mark + PosOffset

// Device Control Entry Offsets

#define kdCtlPosition 10

int Sony_Init (void);
void Sony_Reset (void);

int Check_Sony_Patch (void);
int Check_Sony_Illegal (void);

void Sony_Install (void);
void Sony_Return (int iErr, int SaveD0, int Mode);
void Sony_Insert (UWORD Drive_No, char *filename);
void Sony_Eject (UWORD Drive_No);
void Sony_Access (UWORD Drive_No);
void Sony_Update (void);
void Sony_LEDs (UWORD Drive_No, UBYTE Mode, UWORD Time);

int Sony_Drive_Installed (UWORD Drive_No);
int Sony_Disk_Inserted (UWORD Drive_No);

void Sony_Open (void);
void Sony_Prime (void);
void Sony_Control (void);
void Sony_Status (void);
void Sony_Close (void);

//IRIX CC needs a little new-line at the end of include files.


