// vMac 0.x Source
// By Philip Cummins (PC)
// Copyright PC 1997
   
// PC - VIA Function Files

#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "custom.h"
#include "options.h"
#include "Memory.h"
#include "newcpu.h"
#include "IWM.h"
#include "Keyboard.h"
#include "Mouse.h"
#include "RTC.h"
#include "SCC.h"
#include "Screen.h"
#include "Sound.h"
#include "VIA.h"
#include "VIA_Interface.h"
#	ifdef _VIA_Debug
#include <stdio.h>
#	endif

VIA_Ty VIA;

UBYTE T1_Active = 0;
UBYTE T2_Active = 0;
UBYTE SR_Active = 0;
UBYTE SR_Timer_Active = 0;

ULONG SR_Time = 0;

int  VIA_Init (void)
{
  VIA.ORA   = 0x79; VIA.DDR_A = 0x7F;
  VIA.ORB   = 0x87; VIA.DDR_B = 0x87;
  VIA.T1C_L = VIA.T1C_H = VIA.T1L_L = VIA.T1L_H = 0x00;
  VIA.T2_L  = VIA.T2_H  = VIA.SR    = VIA.ACR   = 0x00;
  VIA.PCR   = VIA.IFR   = VIA.IER   = 0x00;
  T1_Active = T2_Active = SR_Active = 0x00;
  SR_Time   = 0x0000;
  SR_Timer_Active = 0x00;
  return 0;
}

void  VIA_Reset (void)
{
  (void) VIA_Init();
}

void VIA_Interrupt (VIA_Interrupt_Ty VIA_Int)
{

  switch (VIA_Int)
  {
    case T1 :
	  if ((VIA.IER & 0x40) != 0)
	    { VIA.IFR |= 0xC0; }
	  break;
    case T2 :
	  if ((VIA.IER & 0x20) != 0)
	    { VIA.IFR |= 0xA0; }
	  break;
    case CB1 :
	  if ((VIA.IER & 0x10) != 0)
	    { VIA.IFR |= 0x90; }
	  break;
    case CB2 :
	  if ((VIA.IER & 0x08) != 0)
	    { VIA.IFR |= 0x88; }
	  break;
    case SR :
	  if ((VIA.IER & 0x04) != 0)
	    { VIA.IFR |= 0x84; }
	  break;
    case CA1 :
	  if ((VIA.IER & 0x02) != 0)
	    { VIA.IFR |= 0x82; }
	  break;
    case CA2 :
	  if ((VIA.IER & 0x01) != 0)
	    { VIA.IFR |= 0x81; }
      	  break;
    default :
    	  fprintf(stderr, "This is going to crash the system, uncaught VIA.\n");
	  fprintf(stderr, "This is the uncaught interrupt: %d", VIA_Int);
    	  break;
  }
  
  if ((VIA.IFR & 0x80) == 0x80)
    if (regs.intmask <= 1)
      Exception(25);
}

UBYTE VIA_GetB (CPTR addr)
{  
  if ((addr & 1) == 0)
  {
#	ifdef _VIA_Debug
    printf("VIA GB %4lx\n", addr);
#	endif
    switch (addr >> 9)
	{
	  case kORB   :
	    if ((VIA.PCR & 0xE0) == 0)
	      { VIA.IFR &= 0xF7; }
		VIA.ORB = (VIA.ORB & VIA.DDR_B) + VIA_Get_ORB(VIA.DDR_B);
	    return VIA.ORB;   break;
	  case kDDR_B :
	    return VIA.DDR_B; break;
	  case kDDR_A :
	    return VIA.DDR_A; break;
	  case kT1C_L :
	    VIA.IFR &= 0xBF; // Clear T1 Interrupt
	    return VIA.T1C_L; break;
	  case kT1C_H :
	    return VIA.T1C_H; break;
	  case kT1L_L :
	    return VIA.T1L_L; break;
	  case kT1L_H :
	    return VIA.T1L_H; break;
	  case kT2_L  :
	    VIA.IFR &= 0xDF; // Clear T2 Interrupt
	    return VIA.T2_L;  break;
	  case kT2_H  :
	    return VIA.T2_H;  break;
	  case kSR    :
	    if (SR_Active == 1)
	    {
	      switch ((VIA.ACR & 0x1C) >> 2)
	      {
	        case 1 : case 2 : case 3 : // Shifting In
	          VIA.SR = VIA_Get_SR();
	          SR_Time = kSR_Init_Time; // Estimated
	          SR_Timer_Active = 1;
	          SR_Active = 0;
	          break;
	        case 4 : case 5 : case 6 : case 7 : // Shifting Out
	          VIA_Put_SR(VIA.SR);
	          SR_Time = kSR_Init_Time;
	          SR_Timer_Active = 1;
	          SR_Active = 0;
	      }
	    }
	    return VIA.SR;
	    break;
	  case kACR   :
	    return VIA.ACR;   break;
	  case kPCR   :
	    return VIA.PCR;   break;
	  case kIFR   :
	    return VIA.IFR;   break;
	  case kIER   :
	    return VIA.IER;   break;
	  case kORA   :
	  case kORA_H :
	    if ((VIA.PCR & 0xE) == 0)
	      { VIA.IFR &= 0xFE; }
		VIA.ORA = (VIA.ORA & VIA.DDR_A) + VIA_Get_ORA(VIA.DDR_A);
	    return VIA.ORA;   break;
	}
  }
  else
  {
    return 0;
  }
  return 0;
}

UWORD VIA_GetW (CPTR addr)
{
  return 0;
}

ULONG VIA_GetL (CPTR addr)
{
  return 0;
}

void  VIA_PutB (CPTR addr, UBYTE in)
{  
  if ((addr & 1) == 0)
  {
#	ifdef _VIA_Debug
    printf("VIA PB %4lx %2x\n", addr, in);
#	endif
    switch (addr >> 9)
	{
	  case kORB   :
	    if ((VIA.PCR & 0xE0) == 0)
	      { VIA.IFR &= 0xF7; }
		VIA_Put_ORB(VIA.DDR_B,in);
	    VIA.ORB   = in; break;
	  case kDDR_B :
	    VIA.DDR_B = in; break;
	  case kDDR_A :
	    VIA.DDR_A = in; break;
	  case kT1C_L :
	    VIA.IFR &= 0xBF; // Clear T1 Interrupt
	    VIA.T1C_L = in; break;
	  case kT1C_H :
	    VIA.T1C_H = in; break;
	  case kT1L_L :
	    VIA.T1L_L = in; break;
	  case kT1L_H :
	    VIA.T1L_H = in;
	    VIA.IFR &= 0xBF; // Clear T1 Interrupt
	    VIA.T1C_H = VIA.T1L_H;
	    VIA.T1C_L = VIA.T1L_L;
	    T1_Active = 1;
	    break;
	  case kT2_L  :
	    VIA.T2_L  = in; break;
	  case kT2_H  :
	    VIA.IFR &= 0xDF; // Clear T2 Interrupt
	    T2_Active = 1;
	    VIA.T2_H  = in; break;
	  case kSR    :
	    if (SR_Active == 1)
	    {
	      switch ((VIA.ACR & 0x1C) >> 2)
	      {
	        case 1 : case 2 : case 3 : // Shifting In
	          VIA.SR = VIA_Get_SR();
	          SR_Time = kSR_Init_Time; // Estimated
	          SR_Timer_Active = 1;
	          SR_Active = 0;
	          break;
	        case 4 : case 5 : case 6 : case 7 : // Shifting Out
	          VIA.SR = in;
	          VIA_Put_SR(VIA.SR);
	          SR_Time = kSR_Init_Time;
	          SR_Timer_Active = 1;
	          SR_Active = 0;
	      }
	    }   
	    break;
	  case kACR   :
	    VIA.ACR   = in;
	    switch ((VIA.ACR & 0x1C) >> 2)
	    {
	      case 0 :
	        SR_Active = 0; break;
	      default :
	        SR_Active = 1; break;
	    }
	    break;
	  case kPCR   :
	    VIA.PCR   = in; break;
	  case kIFR   :
	    if ((in & 0x80) == 0)
	      { VIA.IFR = VIA.IFR & (0x7F - in); } // Clear Enable Bits
	    else
	      { VIA.IFR = VIA.IFR | (in & 0x7F); } // Set Enable Bits
	    break;
	  case kIER   :
	    if ((in & 0x80) == 0)
	      { VIA.IER = VIA.IER & (0x7F - in); } // Clear Enable Bits
	    else
	      { VIA.IER = VIA.IER | (in & 0x7F); } // Set Enable Bits
	    break;
	  case kORA   :
	  case kORA_H :
	    if ((VIA.PCR & 0xE) == 0)
	      { VIA.IFR &= 0xFE; }
		VIA_Put_ORA(VIA.DDR_A,in);
	    VIA.ORA   = in; break;
	}
  }
}

void  VIA_PutW (CPTR addr, UWORD in)
{
	;
}

void  VIA_PutL (CPTR addr, ULONG in)
{
	;
}

void VIA_Timer (void)
{
  UWORD Temp = 0;
  
  if (SR_Timer_Active == 1)
  {
    SR_Time -= 1;
    if (SR_Time == 0)
    {
      //SR_Timer_Active = 0;
      if ((VIA.IER & 0x04) == 0x04) // Shift Register Interrupts Enabled
        VIA.IFR |= 0x84;
      if ((VIA.IFR & 0x80) == 0x80)
        if (regs.intmask <= 1)
        {
          SR_Timer_Active = 0;
          Exception(25);
        }
    }
  }
  
  if (T1_Active == 1) // Timer 1 is Active
  {
    Temp = (VIA.T1C_H << 8) + VIA.T1C_L; // Get Timer 1 Counter
    Temp -= 1;
    VIA.T1C_H = (Temp & 0xFF00) >> 8; // Save Counter
    VIA.T1C_L = Temp & 0xFF;
    if (Temp == 0) // Complete?
    {
      if ((VIA.ACR & 0x40) == 1) // Free Running?
      {
        VIA.T1C_H = VIA.T1L_H; // Reload Counter from Latches
        VIA.T1C_L = VIA.T1L_L;
      }
      else
      {
        T1_Active = 0; // Deactivate Timer 1
      }
      if ((VIA.IER & 0x40) == 1) // Timer 1 Interrupt Enabled
        VIA.IFR |= 0xC0;
    }
  }
  if (T2_Active == 1)
  {
    Temp = (VIA.T2_H << 8) + VIA.T2_L; // Get Timer 1 Counter
    Temp -= 1;
    VIA.T2_H = (Temp & 0xFF00) >> 8; // Save Counter
    VIA.T2_L = Temp & 0xFF;
    if (Temp == 0)
    {
      T2_Active = 0; // Deactivate Timer 2
      if ((VIA.IER & 0x20) == 1) // Timer 2 Interrupt Enabled
        VIA.IFR |= 0xA0;
    }
  }
  if ((VIA.IFR & 0x60) != 0)
    if (regs.intmask <= 1)
      Exception(25);
}
