// PC - VIA Interface Functions

#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "custom.h"
#include "options.h"
#include "Memory.h"
#include "newcpu.h"
#include "IWM.h"
#include "Keyboard.h"
#include "Mouse.h"
#include "RTC.h"
#include "SCC.h"
#include "Screen.h"
#include "Sound.h"
#include "VIA.h"
#include "VIA_Interface.h"

extern int InterruptNo;

// VIA Interrupt Interface

void VIA_Int (VIA_Int_Ty VIA_Int)
{
  switch (VIA_Int)
  {
 	case Timer_1             : VIA_Interrupt(T1);  break;
	case Timer_2             : VIA_Interrupt(T2);  break;
	case Keyboard_Clock      : VIA_Interrupt(CB1); break;
	case Keyboard_Data       : VIA_Interrupt(CB2); break;
	case Keyboard_Data_Ready : VIA_Interrupt(SR);  break;
	case Vertical_Blanking   : VIA_Interrupt(CA1); break;
	case One_Second          : VIA_Interrupt(CA2); break;
  }
}

// VIA Shift Register Interface

UBYTE VIA_Get_SR (void)
{
  return Keyboard_Get();
}

void  VIA_Put_SR (UBYTE Data)
{
  Keyboard_Put(Data);
}

// VIA_Get_ORA : VIA Get Port A Data
// This function queries VIA Port A interfaced hardware about their status

UBYTE VIA_Get_ORA (UBYTE Selection)
{
  UBYTE Value = 0;
  
  if ((Selection & 0x80) == 0) // SCC Wait/Request
    { Value |= (VIA_GORA7() << 7); }
  
  if ((Selection & 0x40) == 0) // Main/Alternate Screen Buffer
    { Value |= (VIA_GORA6() << 6); }
  
  if ((Selection & 0x20) == 0) // Floppy Disk Line SEL
    { Value |= (VIA_GORA5() << 5); }
  
  if ((Selection & 0x10) == 0) // Overlay/Normal Memory Mapping
    { Value |= (VIA_GORA4() << 4); }
  
  if ((Selection & 0x08) == 0) // Main/Alternate Sound Buffer
    { Value |= (VIA_GORA3() << 3); }
  
  if ((Selection & 0x04) == 0) // Sound Volume Bit 2
    { Value |= (VIA_GORA2() << 2); }
  
  if ((Selection & 0x02) == 0) // Sound Volume Bit 1
    { Value |= (VIA_GORA1() << 1); }
  
  if ((Selection & 0x01) == 0) // Sound Volume Bit 0
    { Value |= VIA_GORA0(); }
  
  return Value;
}

// VIA_Get_ORB : VIA Get Port B Data
// This function queries VIA Port B interfaced hardware about their status

UBYTE VIA_Get_ORB (UBYTE Selection)
{
  UBYTE Value = 0;
  
  if ((Selection & 0x80) == 0) // Sound Enable
    { Value |= (VIA_GORB7() << 7); }
  
  if ((Selection & 0x40) == 0) // Video Beam in Display
    { Value |= (VIA_GORB6() << 6); }
  
  if ((Selection & 0x20) == 0) // Mouse Y2
    { Value |= (VIA_GORB5() << 5); }
  
  if ((Selection & 0x10) == 0) // Mouse X2
    { Value |= (VIA_GORB4() << 4); }
  
  if ((Selection & 0x08) == 0) // Mouse Button
    { Value |= (VIA_GORB3() << 3); }
  
  if ((Selection & 0x04) == 0) // RTC Enable
    { Value |= (VIA_GORB2() << 2); }
  
  if ((Selection & 0x02) == 0) // RTC Data Clock
    { Value |= (VIA_GORB1() << 1); }
  
  if ((Selection & 0x01) == 0) // RTC Data
    { Value |= VIA_GORB0(); }
  
  return Value;
}

void  VIA_Put_ORA (UBYTE Selection, UBYTE Data)
{
  if ((Selection & 0x80) != 0) // SCC Wait/Request
    { VIA_PORA7((Data & 0x80) >> 7); }

  if ((Selection & 0x40) != 0) // Main/Alternate Screen Buffer
    { VIA_PORA6((Data & 0x40) >> 6); }
	
  if ((Selection & 0x20) != 0) // Floppy Disk Line SEL
    { VIA_PORA5((Data & 0x20) >> 5); }
	
  if ((Selection & 0x10) != 0) // Overlay/Normal Memory Mapping
    { VIA_PORA4((Data & 0x10) >> 4); }
	
  if ((Selection & 0x08) != 0) // Main/Alternate Sound Buffer
    { VIA_PORA3((Data & 0x08) >> 3); }
	
  if ((Selection & 0x04) != 0) // Sound Volume Bit 2
    { VIA_PORA2((Data & 0x04) >> 2); }
	
  if ((Selection & 0x02) != 0) // Sound Volume Bit 1
    { VIA_PORA1((Data & 0x02) >> 1); }
	
  if ((Selection & 0x01) != 0) // Sound Volume Bit 0
    { VIA_PORA0(Data & 0x01); }
}
	
void  VIA_Put_ORB (UBYTE Selection, UBYTE Data)
{
  if ((Selection & 0x80) != 0) // Sound Enable
    { VIA_PORB7((Data & 0x80) >> 7); }

  if ((Selection & 0x40) != 0) // Video Beam in Display
    { VIA_PORB6((Data & 0x40) >> 6); }
	
  if ((Selection & 0x20) != 0) // Mouse Y2
    { VIA_PORB5((Data & 0x20) >> 5); }
	
  if ((Selection & 0x10) != 0) // Mouse X2
    { VIA_PORB4((Data & 0x10) >> 4); }
	
  if ((Selection & 0x08) != 0) // Mouse Button
    { VIA_PORB3((Data & 0x08) >> 3); }

  if ((Selection & 0x04) != 0) // RTC Enable
    { VIA_PORB2((Data & 0x04) >> 2); }
	
  if ((Selection & 0x02) != 0) // RTC Data Clock
    { VIA_PORB1((Data & 0x02) >> 1); }
	
  if ((Selection & 0x01) != 0) // RTC Data
    { VIA_PORB0(Data & 0x01); }
}
