/*
 * vMac - Screen-Motif.c
 * Xt/Lesstif(Motif) X Window Interface.
 * Copyright 1997 Michael Hanni (mhanni@sprintmail.com)
 */

#include "sysconfig.h"
#include "sysdeps.h"
#include "config.h"
#include "Screen.h"
#include "RAM.h"  
#include "VIA.h"
#include "Keyboard.h"
#include "Sony.h"
#include "prefs.h"
#include "debug.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/PushB.h>
#include <Xm/MenuShell.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>
#include <Xm/CascadeB.h>
#include <Xm/MessageB.h>
#include <Xm/FileSB.h>
#include <Xm/SelectioB.h>

#include "Screen-Xt.h"

#include <stdio.h>

#if SCAN
#if MITSHM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif
#endif

// Global values.

Display 	*display;
XColor          black, white;
Drawable	window;
GC              vmacgc;
Visual          *Xvisual;
int             Xdepth;
Colormap        Xcmap;
int             macDisplayHeight = 342;
int             macDisplayWidth = 512;
Screen		*Xscreen;
Widget 		field, drive1, drive2;
Widget		exitbutton, restorebutton;
XtAppContext 	app;

int		lastmx = 0, lastmy = 0;
int 		buttonstate[3];
int		inwindow;

KeySym 		Key, Key2 = 0x01C, Key3 = 0x01D, Key4 = 0x01E, Key5 = 0x01F;
char 		ourKey;
UWORD 		Modifier;

#if SCAN
#if MITSHM
XShmSegmentInfo shminfo;
#endif
char          	*pixmapBuffer;
UWORD         	*previousScreen;
XImage        	*offscreenBuffer;
#else
XImage          *vmacimg;
char            *image_Mem1;
#endif


UBYTE vPage2 = 1;

// Prototypes.

inline void Catch_X_Events(void);
void Quit_vMac(void);
void MC68000_reset(void);
void MC68000_step(void);
void MC68000_run(void);

void exitCall(Widget w, XtPointer client, XtPointer data)
{
	Quit_vMac();
}

void resetCall(Widget w, XtPointer client, XtPointer data)
{
	MC68000_reset();
}

void Cancel(Widget w, XtPointer client, XtPointer data)
{
	XtUnmanageChild(w);
}

void SelectedFile(Widget w, caddr_t client_data,
			XmFileSelectionBoxCallbackStruct *selection) {

	char *filename;
	
	XmStringGetLtoR(selection->value, XmSTRING_DEFAULT_CHARSET, &filename);
	
	Sony_Insert(0x001, filename);

	strcpy(thePrefs.DrivePath[0], filename);

	XtUnmanageChild(w);

}

void SelectedFile2(Widget w, caddr_t client_data,
			XmFileSelectionBoxCallbackStruct *selection) {

	char *filename;
	
	XmStringGetLtoR(selection->value, XmSTRING_DEFAULT_CHARSET, &filename);
	
	Sony_Insert(0x002, filename);

	strcpy(thePrefs.DrivePath[1], filename);

	XtUnmanageChild(w);

}

inline void keypress_eh(Widget w, XtPointer data, XKeyEvent *xkey) 
{
	int j;
	j = XLookupString(xkey, &ourKey, 1, &Key, NULL);
	switch (Key) {
		case XK_Left:
			Keyboard_Down(Key2);
			break;
		case XK_Right:
			Keyboard_Down(Key3);
			break;
		case XK_Up:
			Keyboard_Down(Key4);
			break;
		case XK_Down:
			Keyboard_Down(Key5);
			break;
		case XK_Meta_L:
		case XK_Meta_R:
		case XK_Alt_L:
		case XK_Alt_R:
		case XK_Control_L:
		case XK_Control_R:
		case XK_Shift_L:
		case XK_Shift_R:
		case XK_Caps_Lock:
			break;
		default:
			if (xkey->state & ShiftMask)
				Modifier = 0x0200;
			else if (xkey->state & ControlMask)
				Modifier = 0x0100;
			else if (xkey->state & Mod1Mask)
				Modifier = 0x0800;
			else if (xkey->state & LockMask)
				Modifier = 0x0400;
			else
				Modifier = 0x0000;
			PC_PostEvent (3, Key, Modifier);
			break;
	}
}


inline void button_press(Widget w, XtPointer data, XButtonEvent *event)
{
       	if (inwindow) {
	buttonstate[event->button-1] = 1;
	}
}

inline void button_release(Widget w, XtPointer data, XButtonEvent *event)
{
       	if (inwindow) {
	buttonstate[event->button-1] = 0;
	}
}

inline void mouse_motion(Widget w, XtPointer data, XMotionEvent *event) 
{
       	if (inwindow) {
        	lastmx = event->x;
              	lastmy = event->y;
        }
}

inline void enter_notify(Widget w, XtPointer data, XCrossingEvent *event)
{
	lastmx = event->x;
	lastmy = event->y;
      	inwindow = 1;
}

inline void leave_notify(Widget w, XtPointer data, XCrossingEvent *event) 
{
      	inwindow = 0;
}

inline void keyrelease_eh(Widget w, XtPointer data, XKeyEvent *xkey)
{
	int j;
	j = XLookupString(xkey, &ourKey, 1, &Key, NULL);
	switch (Key) {
		case XK_Left:
			Keyboard_Up(Key2);
			break;
		case XK_Right:	
			Keyboard_Up(Key3);
			break;
		case XK_Up:
			Keyboard_Up(Key4);
			break;
		case XK_Down:
			Keyboard_Up(Key5);
			break;
		case XK_Meta_L:
		case XK_Meta_R:
		case XK_Alt_L:
		case XK_Alt_R:
		case XK_Control_L:
		case XK_Control_R:
		case XK_Shift_L:
		case XK_Shift_R:
		case XK_Caps_Lock:
			break;
		default:
			if (xkey->state & ShiftMask)
				Modifier = 0x0200;
			else if (xkey->state & ControlMask)
				Modifier = 0x0100;
			else if (xkey->state & Mod1Mask)
				Modifier = 0x0800;
			else if (xkey->state & LockMask)
				Modifier = 0x0400;
			else
				Modifier = 0x0000;
			PC_PostEvent (4, Key, Modifier);
			break;
	}
}        

void Error_popup(Widget w, char *text,
			XmPushButtonCallbackStruct *cbs) {

	Widget dialog, remove, remove2;
	XmString string, title;
	Arg wargs[3];
	int n;
	
	string = XmStringCreateSimple(text);
	title = XmStringCreateSimple("vMac Warning!");

	n = 0;
	XtSetArg(wargs[n], XmNmessageString, string); n++;
	XtSetArg(wargs[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	XtSetArg(wargs[n], XmNdialogTitle, title); n++;
	
	dialog = XmCreateWarningDialog(w, "warning", wargs, n);

	if (w == exitbutton)
		XtAddCallback(dialog, XmNokCallback, exitCall, NULL);
	else if (w == restorebutton)
		XtAddCallback(dialog, XmNokCallback, resetCall, NULL);
	else {
		XtAddCallback(dialog, XmNokCallback, Cancel, NULL);
		remove2 = XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON);
		XtUnmanageChild(remove2);
	}		

	remove = XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON);
	XtUnmanageChild(remove);

	XmStringFree(string);
	XmStringFree(title);
	
	XtManageChild(dialog);
	XtPopup(XtParent(dialog), XtGrabNone);
	
}	

void About_popup(Widget w, char *text,
			XmPushButtonCallbackStruct *cbs) {

	Widget dialog, remove, remove2;
	XmString title;
	Arg wargs[3];
	int n;

	title = XmStringCreateSimple("vMac About (Nothing yet!)");

	n = 0;
	XtSetArg(wargs[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
	XtSetArg(wargs[n], XmNdialogTitle, title); n++;

	dialog = XmCreateMessageDialog(w, "info", wargs, n);

	XtAddCallback(dialog, XmNokCallback, Cancel, NULL);

	remove = XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON);
	remove2 = XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON);
	XtUnmanageChild(remove);
	XtUnmanageChild(remove2);

	XtManageChild(dialog);
	XtPopup(XtParent(dialog), XtGrabNone);

}

void FileSelect_popup(Widget w, XtPointer clientdata,
			XmPushButtonCallbackStruct *cbs) {

	Widget dialog, remove;
	XmString mask, title;
	Arg wargs[3];
	int n, driveNum = 0;

	if (w == drive1) 
		driveNum = 1;
	else if (w == drive2)
		driveNum = 2;
	else
		fatal_error("error in vMac drive numbering. (FileSelect_popup.)");

	if (Sony_Disk_Inserted(driveNum) == 0) {
		
		n = 0;	
		mask = XmStringCreateSimple("*.c");
		title = XmStringCreateSimple("Select disk...");

		XtSetArg(wargs[n], XmNdefaultButtonType, XmDIALOG_CANCEL_BUTTON); n++;
		XtSetArg(wargs[n], XmNdialogTitle, title); n++;

		dialog = XmCreateFileSelectionDialog(w, "disk", wargs, n);
		XtAddCallback(dialog, XmNcancelCallback, Cancel, NULL);
		
		if (driveNum == 1)
			XtAddCallback(dialog, XmNokCallback, (void *)SelectedFile, NULL);
		else if (driveNum == 2)
			XtAddCallback(dialog, XmNokCallback, (void *)SelectedFile2, NULL);
		else
			fatal_error("error in vMac drive numbering. (FileSelect_popup.)");

		XmStringFree(mask);
		XmStringFree(title);

		remove = XmSelectionBoxGetChild(dialog, XmDIALOG_HELP_BUTTON);
		XtUnmanageChild(remove);
	
		XtManageChild(dialog);
		XtPopup(XtParent(dialog), XtGrabNone);
	}
	else {
		Error_popup(w, "Drive not empty, eject the disk.", NULL);
	} 

}

int Menubar_Init(Widget w) {

	Widget menubar;
	Widget filemenu, diskmenu, helpmenu;
	Widget filecascade, diskcascade, helpcascade;
	Widget about;
	Arg wargs[10];
	int n;

	menubar = XmCreateMenuBar(w, "menubar", NULL, 0);
  
	/* Create the "File" menu */

  	filemenu = XmCreatePulldownMenu(menubar, "filemenu", NULL, 0);
      	XtManageChild(filemenu);
  
    	n = 0;
      	XtSetArg (wargs[n], XmNsubMenuId, filemenu);  n++;
       	filecascade = XmCreateCascadeButton (menubar, "File", wargs, n);
      	XtManageChild (filecascade);

     	restorebutton = XmCreatePushButton (filemenu, "Reset", NULL, 0);
	XtAddCallback(restorebutton, XmNactivateCallback, (void *) Error_popup, "Really reset?");
      	XtManageChild(restorebutton);

     	exitbutton = XmCreatePushButton (filemenu, "Exit", NULL, 0);
	XtAddCallback(exitbutton, XmNactivateCallback, (void *) Error_popup, "Really quit?");
      	XtManageChild(exitbutton);

	/* Create the "Disks" menu */

  	diskmenu = XmCreatePulldownMenu(menubar, "diskmenu", NULL, 0);
      	XtManageChild(diskmenu);

    	n = 0;
      	XtSetArg (wargs[n], XmNsubMenuId, diskmenu);  n++;
       	diskcascade = XmCreateCascadeButton (menubar, "Disks", wargs, n);
      	XtManageChild(diskcascade);
          
     	drive1 = XmCreatePushButton (diskmenu, "Drive 1", NULL, 0);
	XtAddCallback(drive1, XmNactivateCallback, (void *) FileSelect_popup, NULL);
      	XtManageChild(drive1);

     	drive2 = XmCreatePushButton (diskmenu, "Drive 2", NULL, 0);
	XtAddCallback(drive2, XmNactivateCallback, (void *) FileSelect_popup, NULL);
      	XtManageChild(drive2);
                  
	/* Help menu */

  	helpmenu = XmCreatePulldownMenu(menubar, "helpmenu", NULL, 0);
      	XtManageChild(helpmenu);

    	n = 0;
      	XtSetArg (wargs[n], XmNsubMenuId, helpmenu);  n++;
       	helpcascade = XmCreateCascadeButton (menubar, "Help", wargs, n);
      	XtManageChild(helpcascade);

     	about = XmCreatePushButton (helpmenu, "About", NULL, 0);
	XtAddCallback(about, XmNactivateCallback, (void *) About_popup, NULL);
      	XtManageChild(about);

  	XtManageChild(menubar);

	return 0;
}

int Graphics_Setup(Widget parent) {

	display = XtDisplay(parent);
	Xscreen = XtScreen(parent);
	Xdepth = DefaultDepthOfScreen(Xscreen);
	Xvisual = DefaultVisualOfScreen(Xscreen);
	Xcmap = DefaultColormapOfScreen(Xscreen);
        white.pixel = WhitePixelOfScreen(Xscreen);
	XQueryColor(display, Xcmap, &white);
        black.pixel = BlackPixelOfScreen(Xscreen);
        XQueryColor(display, Xcmap, &black);	
        
	return 0;
}

int Graphics_Init(Widget drawable) {

        vmacgc = XtGetGC(drawable, 0, NULL);
        XSetState(display, vmacgc, black.pixel, white.pixel, GXcopy, AllPlanes);

#if SCAN
	previousScreen = calloc(1,(512/8)*384 ); /* Ugh. */
#if MITSHM
	fprintf(stderr, "Using MIT-SHaredMemory Extensions.\n");
	offscreenBuffer = XShmCreateImage(display, Xvisual, Xdepth, ZPixmap,
	NULL, &shminfo, 512, 384);
    	shminfo.shmid = shmget(IPC_PRIVATE, 4*512*384, IPC_CREAT|0777);
	shminfo.shmaddr = offscreenBuffer->data = (char *)shmat(shminfo.shmid,
	0, 0);
	shminfo.readOnly = False;
    	XShmAttach(display, &shminfo);
        XSync(display, 0);
#else
	pixmapBuffer = malloc( 4*512*384 ); /* Ugh. Fix */
	offscreenBuffer = XCreateImage( display, Xvisual, Xdepth, ZPixmap, 0,
		(char *) pixmapBuffer, 512, 384, (Xdepth==8)?8:32, 0);
#endif
#else

      	image_Mem1 = (char *)calloc(1, 64 * macDisplayHeight);
    	vmacimg = XCreateImage(display, Xvisual, 1, XYBitmap, 0, (char *)image_Mem1, 
    		macDisplayWidth, macDisplayHeight, 32, 512 / 8);
      	vmacimg->bitmap_bit_order = MSBFirst;
     	vmacimg->byte_order = MSBFirst;
#endif

	return 0;
	
}

int Screen_Init(int argc, char **argv) {

	Widget toplevel, mainWin;
        XrmDatabase database;

	toplevel = XtVaAppInitialize(&app, "vMac", NULL, 0, &argc, argv, NULL, NULL);

	Graphics_Setup(toplevel);

        database = XrmGetDatabase(display);
       	XrmPutStringResource(&database, "*background", "#c4c4c4");
       	XrmPutStringResource(&database, "*fontList", "-*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*");
        XrmSetDatabase(display, database);

	mainWin = XtVaCreateManagedWidget("MainWin", xmMainWindowWidgetClass,
			toplevel, NULL);

	Menubar_Init(mainWin);

	field = XtVaCreateManagedWidget("drawField",
		xmDrawingAreaWidgetClass, mainWin, XmNheight,
		macDisplayHeight, XmNwidth, macDisplayWidth, NULL);

	Graphics_Init(field);	

	XtAddEventHandler (field, KeyPressMask, FALSE, 
			  (XtEventHandler)keypress_eh, NULL);
	XtAddEventHandler (field, KeyReleaseMask, FALSE, 
			  (XtEventHandler)keyrelease_eh, NULL);
	XtAddEventHandler (field, ButtonPressMask, FALSE, 
			  (XtEventHandler)button_press, NULL);
	XtAddEventHandler (field, ButtonReleaseMask, FALSE, 
			  (XtEventHandler)button_release, NULL);
	XtAddEventHandler (field, PointerMotionMask, FALSE, 
			  (XtEventHandler)mouse_motion, NULL);
	XtAddEventHandler (field, EnterWindowMask, FALSE, 
			  (XtEventHandler)enter_notify, NULL);
	XtAddEventHandler (field, LeaveWindowMask, FALSE, 
			  (XtEventHandler)enter_notify, NULL);

	XtRealizeWidget(toplevel);

	return 0;

}

#if SCAN

#define NotChanged -1

inline void Screen_Draw(void) {

	UWORD  *screen;
  	int     x, y;
  	int     bitTest, bitPos;
  	int     currentLineChanged;
  	UWORD   currentWord;
  	UWORD   previousWord;
  	int     firstLineChanged, lastLineChanged;

  	window = XtWindow(field);

  	if (vPage2 == 1)
    		screen = (UWORD *) RAM_Xlate(kMain_Buffer);
  	else
    		screen = (UWORD *) RAM_Xlate(kAlternate_Buffer);

  	firstLineChanged = lastLineChanged = NotChanged;

  	for( y = 0; y < 384; y++ )
    	{
    		currentLineChanged = 0;
    		for( x = 0; x < 512/16; x++ )
      		{
      			currentWord = screen[x + (y * 512/16)];
      			previousWord = previousScreen[x + (y * 512/16)];
      			if( currentWord != previousWord )
        		{
        			bitTest = 1 << (16 - 1);
        			for( bitPos = 0; bitPos < 16; bitPos++ )
          			{
          				if( bitTest & currentWord )
            					XPutPixel( offscreenBuffer, (x*16)+bitPos, y, black.pixel );
          				else
            					XPutPixel( offscreenBuffer, (x*16)+bitPos, y, white.pixel );
          				bitTest = bitTest >> 1;
          			}
        			currentLineChanged = 1;
        		}
      		}
    	if( currentLineChanged == 1 )
      		if( firstLineChanged == NotChanged ) 
        		firstLineChanged = lastLineChanged = y;
      		else
        		lastLineChanged = y;
    	}

  	if( firstLineChanged != NotChanged )
    	{
    		memcpy( previousScreen, screen, (512/8)*384 );
#if MITSHM
    		XShmPutImage( display, window, vmacgc, offscreenBuffer, 0,
    				firstLineChanged, 0, firstLineChanged, 512,
				lastLineChanged - firstLineChanged + 1, 0);
#else
    		XPutImage(display, window, vmacgc, offscreenBuffer, 0, firstLineChanged,
      				0, firstLineChanged, 512, lastLineChanged - firstLineChanged + 1);
#endif   
    	}

  Catch_X_Events();
}
#else
inline void Screen_Draw(void)
{
	char 	*dst =(char *)vmacimg->data;
	char	*src;
	int	i;

	window = XtWindow(field);

	if (vPage2 == 1)
		(char *)src = (char *)RAM_Xlate(kMain_Buffer);
	else
		(char *)src = (char *)RAM_Xlate(kAlternate_Buffer);
	
	for (i = 0; i < macDisplayWidth*macDisplayHeight/8/2; i++)
	{
#	ifdef FIXEND
		*(dst++) = *(src+1);
		*(dst++) = *(src);
#	else
		*(dst++) = *(src);
		*(dst++) = *(src+1);
#	endif
		src+=2;
	}
	XPutImage(display, window, vmacgc, vmacimg, 0, 0, 0, 0,
	       	  vmacimg->width, vmacimg->height);

	Catch_X_Events();
}
#endif

void Screen_Reset(void)
{
	;
}

inline void Catch_X_Events(void)
{
   	int i = 0;
   	XEvent event;  

   	while(XtAppPending(app) > 0 && i < 10)
   	{
	     	XtAppNextEvent(app, &event);
		XtDispatchEvent(&event);
   		i++;
   	}
}

// VIA Interface Functions

UBYTE VIA_GORA6 (void) // Main/Alternate Screen Buffer
{
#	ifdef _VIA_Interface_Debug
  printf("VIA ORA6 attempts to be an input\n");
#	endif
  return 0;
}

void  VIA_PORA6 (UBYTE Data)
{
  vPage2 = Data;
}

UBYTE VIA_GORB6 (void) // Video Beam in Display
{
  return 0; // Assume it is
}

void  VIA_PORB6 (UBYTE Data)
{
#	ifdef _VIA_Interface_Debug
  printf("VIA ORB6 attempts to be an output\n");
#	endif
}
